/* 
 *  "evkal.c" - modul evangelickeho kalendare pro php3.
 *
 */

#include "evkal.h"

/* 
 * this array should be set up as:
 * {"PHPScriptFunctionName",dllFunctionName,1} 
 */

function_entry evkal_functions[] = {
	{"jd2gred", evkal_jd2gred, NULL},
	{"jd2grem", evkal_jd2grem, NULL},
	{"jd2grey", evkal_jd2grey, NULL},
	{"gre2jd", evkal_gre2jd, NULL},

	{"ekdow", evkal_dow, NULL},

	{"eksnames", evkal_gre_s, NULL},
	{"eksunnames", evkal_kal_sun, NULL},

	{"ekgs2jd", evkal_gs_jd, NULL},
	{"ekgstojd", evkal_gs_jd, NULL},
	{"ekjd2gs", evkal_jd_gs, NULL},
	{"ekjdtogs", evkal_jd_gs, NULL},
	{"eksmajor", evkal_gs_major, NULL},

	{"ekjd2gsun", evkal_jd_gsun, NULL},
	{"ekjdtogsun", evkal_jd_gsun, NULL},
	{"eklasttrin", evkal_last_trin, NULL},
	{"eklastepif", evkal_last_epif, NULL},
	{"ekgsun2jd", evkal_gsun_jd, NULL},
	{"ekgsuntojd", evkal_gsun_jd, NULL},

	{"ekcopyright", evkal_copy, NULL},
	{NULL, NULL, NULL}
};


php3_module_entry evkal_module_entry = {
	"Evkal", evkal_functions, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL
};


#if COMPILE_DL
DLEXPORT php3_module_entry *get_module(void) { return &evkal_module_entry; }
#endif

/* ========== Pomocne funkce:
 * otestuje meze a kdyz jsou hodnoty mimo, nastavi je na meze...
 * (to jen nahodou, kdyby se nekdo pokousel o nejaky overflow...)
 * ... at zije paranoia !!!
 */

inline void c_jd (long *jd) 
{
 if (*jd<1000000) *jd=1000000;
 if (*jd>5000000) *jd=5000000;
}

inline void c_y (long *y)
{
 if (*y<-1000) *y=-1000;
 if (*y>9000) *y=9000;
}

inline void c_m (long *m)
{
 if (*m<1) *m=1;
 if (*m>12) *m=12;
}

inline void c_d (long *d)
{
 if (*d<1) *d=1;
 if (*d>31) *d=31;
}

inline void c_i (long *i)
{
 if (*i<0) *i=0; /* i==0 znamena "zadny svatek/nedele" */
 if (*i>70) *i=70; /* to by melo snad stacit, ne?... */
}

/* ========== Kalendarni funce =============== */

/* {{{ proto int jd2gred(long jd)
    jd ->> d */
DLEXPORT void evkal_jd2gred(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *jd;
        int d,m,y;

        if (getParameters(ht, 1, &jd) == SUCCESS) {
                convert_to_long(jd);
		c_jd(&(jd->value.lval));

                kal_conv_jd_gre(jd->value.lval, &d, &m, &y);

                return_value->value.lval = d;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}

/* {{{ proto int jd2grem(long jd)
    jd ->> m */
DLEXPORT void evkal_jd2grem(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *jd;
        int d,m,y;

        if (getParameters(ht, 1, &jd) == SUCCESS) {
                convert_to_long(jd);
		c_jd(&(jd->value.lval));

                kal_conv_jd_gre(jd->value.lval, &d, &m, &y);

                return_value->value.lval = m;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}

/* {{{ proto int jd2grey(long jd)
    jd ->> y */
DLEXPORT void evkal_jd2grey(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *jd;
        int d,m,y;

        if (getParameters(ht, 1, &jd) == SUCCESS) {
                convert_to_long(jd);
		c_jd(&(jd->value.lval));

                kal_conv_jd_gre(jd->value.lval, &d, &m, &y);

                return_value->value.lval = y;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}

/* {{{ proto int gre2jd(long jd)
    j,d,m ->> jd */
DLEXPORT void evkal_gre2jd(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *d, *m, *y;
        int jd;

        if (getParameters(ht, 3, &d, &m, &y) == SUCCESS) {
                convert_to_long(d);
                convert_to_long(m);
                convert_to_long(y);
		c_d(&(d->value.lval));
		c_m(&(m->value.lval));
		c_y(&(y->value.lval));

                jd= kal_conv_gre_jd
	((int) d->value.lval, (int) m->value.lval,  (int) y->value.lval);

                return_value->value.lval = jd;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}

/* {{{ proto int EKdow(long jd)
     day of week */
DLEXPORT void evkal_dow(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *jd;
        int d;

        if (getParameters(ht, 1, &jd) == SUCCESS) {
                convert_to_long(jd);
		c_jd(&(jd->value.lval));

                d= kal_day_of_week(jd->value.lval);

                return_value->value.lval = d;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}



/* ============== Nazvy ====================== */

/* {{{ proto string EKgsnames(int jd)
    Vrati jmeno svatku */
DLEXPORT void evkal_gre_s(INTERNAL_FUNCTION_PARAMETERS)
{
	pval * i;
	char * name;

	if (getParameters(ht, 1, &i) == SUCCESS) {
		convert_to_long(i);
		c_i(&(i->value.lval));
		name=kal_gre_s((int) i->value.lval);

		return_value->value.str.val = estrdup(name);
		return_value->value.str.len = strlen(name);
		return_value->type = IS_STRING;
	} else {
		WRONG_PARAM_COUNT;
	}
	return;
}

/* {{{ proto string EKgsunnames(int jd)
    Vrati jmeno nedele */

DLEXPORT void evkal_kal_sun(INTERNAL_FUNCTION_PARAMETERS)
{
        pval * i;
        char * name;

        if (getParameters(ht, 1, &i) == SUCCESS) {
                convert_to_long(i);
		c_i(&(i->value.lval));
                name=kal_sun((int) i->value.lval);

                return_value->value.str.val = estrdup(name);
                return_value->value.str.len = strlen(name);
                return_value->type = IS_STRING;
        } else {
                WRONG_PARAM_COUNT;
        }
        return;
}

/*================== Svatky ===================*/

/* {{{ proto int evkal_gs_jd(int num, int year)
   Cislo svatku ->> julianske datum */
DLEXPORT void evkal_gs_jd(INTERNAL_FUNCTION_PARAMETERS)
{
	pval *num, *year;
	int jd;

	if (getParameters(ht, 2, &num, &year) == SUCCESS) {
		convert_to_long(num);
		c_i(&(num->value.lval));
		convert_to_long(year);
		c_y(&(year->value.lval));

  	    jd = kal_conv_gs_jd((int) num->value.lval,(int) year->value.lval);

		return_value->value.lval = jd;
		return_value->type = IS_LONG;
	} else {
		WRONG_PARAM_COUNT;
	}
}

/* {{{ proto int evkal_jd_gs(long jd)
   Julianske datum ->> cislo svatku */
DLEXPORT void evkal_jd_gs(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *jd;
        int num;

        if (getParameters(ht, 1, &jd) == SUCCESS) {
                convert_to_long(jd);
		c_jd(&(jd->value.lval));

		num = (int) kal_conv_jd_gs( jd->value.lval);

                return_value->value.lval = num;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}


/* {{{ proto int evkal_gs_major(int num)
    Je to hlavni svatek? */
DLEXPORT void evkal_gs_major(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *num;
        int bol;

        if (getParameters(ht, 1, &num) == SUCCESS) {
                convert_to_long(num);
		c_i(&(num->value.lval));

                bol = kal_gs_major((int) num->value.lval);

                return_value->value.lval = bol;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}

/*================== Nedele ===================*/

/* {{{ proto int evkal_gsun_jd(int num, int year)
   Cislo nedele ->> julianske datum */
DLEXPORT void evkal_gsun_jd(INTERNAL_FUNCTION_PARAMETERS)
{
	pval *num, *year;
	int jd;

	if (getParameters(ht, 2, &num, &year) == SUCCESS) {
		convert_to_long(num);
		c_i(&(num->value.lval));
		convert_to_long(year);
		c_y(&(year->value.lval));

  	    jd = kal_conv_gsun_jd((int) num->value.lval,(int) year->value.lval);

		return_value->value.lval = jd;
		return_value->type = IS_LONG;
	} else {
		WRONG_PARAM_COUNT;
	}
}

/* {{{ proto int evkal_jd_gsun(long jd)
   Julianske datum ->> cislo nedele */
DLEXPORT void evkal_jd_gsun(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *jd;
        int num;

        if (getParameters(ht, 1, &jd) == SUCCESS) {
                convert_to_long(jd);
		c_jd(&(jd->value.lval));

		num = (int) kal_conv_jd_gsun( jd->value.lval);

                return_value->value.lval = num;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}

/* {{{ proto int EKLastTrin(int year)
    Posledni nedele po trojici (cislo) */
DLEXPORT void evkal_last_trin(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *year;
        int num;

        if (getParameters(ht, 1, &year) == SUCCESS) {
                convert_to_long(year);
		c_y(&(year->value.lval));

                num = kal_last_trin((int) year->value.lval);

                return_value->value.lval = num;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}

/* {{{ proto int evkal_last_trin(int year)
    Posledni nedele po epifanii (cislo) */
DLEXPORT void evkal_last_epif(INTERNAL_FUNCTION_PARAMETERS)
{
        pval *year;
        int num;

        if (getParameters(ht, 1, &year) == SUCCESS) {
                convert_to_long(year);
		c_y(&(year->value.lval));

                num = kal_last_epif((int) year->value.lval);

                return_value->value.lval = num;
                return_value->type = IS_LONG;
        } else {
                WRONG_PARAM_COUNT;
        }
}

DLEXPORT void evkal_copy(INTERNAL_FUNCTION_PARAMETERS)
{
 php3_printf("(c) 1996-2000 <a href=\"http://www.etf.cuni.cz/~tomasek/\">Petr Tomek</a>\n");
}

